--[[
LuCI - Lua Configuration Interface for MeshCruzer
Copyright 2010 Kentarou Matsuyama <matsuyama@thinktube.com>

]]--
local util = require "luci.util"
local sys  = require "luci.sys"
local ttu  = require "luci.tools.ttutil"
local web  = require "luci.tools.webadmin"
local nw   = require "luci.tools.network"
local lip  = require "luci.ip"

local adhoc_list = ttu.get_ifaces4adhoc()
local exsubnet_list = ttu.get_ifaces4exsubnet()

----------- helper function --------------
local function get_exentry_by_network(network)
	for _, v in pairs(exsubnet_list) do
		if v.netname == network then
			return v
		end
	end
	return nil
end

local function get_exentry_by_ipaddr(ipaddr)
	for _, v in pairs(exsubnet_list) do
		local ip = lip.IPv4(v.ipaddr, v.netmask)
		if ip and ip:network():string() == ipaddr then
			return v
		end
	end
end
-------------------------------------------

m = Map("meshz", translate("mesh_index"), translate("mesh_index_desc"))

---- main parameters
main = m:section(TypedSection, "configuration", translate("mz_mainconf"), "")
main.anonymous = true

name = main:option(Value, "node_name", translate("mz_nodename"))
name.rmempty = false

mz_ipaddr = main:option(Value, "ip_address", translate("mz_ipaddr"))
mz_ipaddr.rmempty = false

mz_subnet = main:option(Value, "adhoc_subnet", translate("mz_subnet"))
mz_subnet.rmempty = false

meshgw = main:option(ListValue, "mesh_gateway", translate("mz_meshgw"))
meshgw:value("yes", translate("enable"))
meshgw:value("no", translate("disable"))

-- adhoc interfaces
intf = main:option(MultiValue, "adhoc_interface", translate("mz_adhoc_intf"))
intf.rmempty = false
intf.widget = "checkbox"
for k, v in ipairs(adhoc_list) do
	if v.ifname and v.netname then
		intf:value(v.ifname, v.netname .. " (" .. v.ifname .. ")" )
	end
end

--[[
fixip = main:option(Flag, "fixip", translate("mz_syncip", "Sync above configuration to Network Configuration"))
fixip.enabled = "1"

function fixip.write(self, section, value)
	local ip = self.map.uci:get("meshz", section, "ip_address") or ""
	local mask = self.map.uci:get("meshz", section, "adhoc_subnet") or ""
	local ifaces = self.map.uci:get("meshz", section, "adhoc_interface") or ""

	if value == "1" and #ip > 0 and #mask > 0 and #ifaces > 0 then
		m:chain_top("network")

		-- convert to table
		if type(ifaces) ~= "table" then
			ifaces = ttu.split(ifaces, " ")
		end

		-- create address and netmask
		mask = mask:match("/(.+)")
		ip = lip.IPv4(ip.."/"..mask)

		for _, i in pairs(ifaces) do
			local netname
			-- if the device is bridge interface like "br-LAN", delete "br-" suffix
			if string.find(i, "br-") then
				netname = string.gsub(i, "br%-", "")
			elseif string.match(i, "^wlan%d+") then
				self.map.uci:foreach("wireless", "wifi-iface",
					function(s)
						if s.ifname == i then
							netname = s.network
						end
					end)
			else
				netname = web.iface_get_network(i)
			end

			if netname then
				-- chain ip address to network configuration
				m.uci:tset("network", netname, {
					proto   = "static",
					ipaddr  = ip:host():string(),
					netmask = ip:mask():string(),
				})
				-- make sure there is no defaultroute, peerdns and gateway
				m.uci:delete("network", netname, "defaultroute")
				m.uci:delete("network", netname, "peerdns")
				m.uci:delete("network", netname, "gateway")
			end
		end
		m.uci:save("network")
	end
end
--]]

---- link parameter
link = m:section(TypedSection, "link", translate("mz_link_param"))
link.anonymous = true

min_rate = link:option(Value, "link_min_rate", translate("mz_min_rate"))
min_rate.rmempty = true
min_rate.optional = true

min_rssi = link:option(Value, "link_min_rssi", translate("mz_min_rssi"))
min_rssi.rmempty = true

pref_rate = link:option(Value, "link_pref_rate", translate("mz_pref_rate"))
pref_rate.rmemtpy = true
pref_rate.optional = true

pref_rssi = link:option(Value, "link_pref_rssi", translate("mz_pref_rssi"))
pref_rssi.rmempty = true

---- routing
route = m:section(TypedSection, "routing", translate("mz_extsubnet"), translate("mz_extsubnet_desc"))
route.addremove = true
route.anonymous = true
route.cbi_add_title = translate("mz_add_fsnet", "Add New FlatSubnet")
route.cbi_del_title = translate("mz_del_fsnet", "Delete FlatSubnet")

-- external subnet definition
fs_subnet = route:option(Value, "flat_stub_subnet", translate("mz_flat_ss"))
fs_subnet.rmempty = true
-- add address from default value
m.uci:foreach("meshz", "routing",
	function(s)
		if s.flat_stub_subnet and #s.flat_stub_subnet > 0 then
			fs_subnet:value(s.flat_stub_subnet)
		end
	end)
-- add address from external subnet list
for _, v in ipairs(exsubnet_list) do
	local ip = lip.IPv4(v.ipaddr, v.netmask)
	if ip then
		local entry = ip:network():string().."/"..ip:prefix()
		if v.ifname == "alias" then
			fs_subnet:value(entry, entry .. " (" .. v.netname .. "-alias)")
		else
			fs_subnet:value(entry, entry .. " (" .. v.netname .. ")")
		end
	end
end

-- virtual gateway ip address
fs_gwip = route:option(Flag, "flat_subnet_gw_ip_enable", translate("mz_flat_gwip"))
fs_gwip.enabled = 'yes'
fs_gwip.disabled = 'no'
fs_gwip.rmempty = true

-- fast roaming feature
fs_vmac = route:option(Flag, "flat_subnet_gw_virtual_mac_enable", translate("mz_flat_vmac"))
fs_vmac.enabled = 'yes'
fs_vmac.disabled = 'no'
fs_vmac.rmempty = true
fs_vmac:depends("flat_subnet_gw_ip_enable", "1")

---- manager
mgr = m:section(TypedSection, "manager", translate("mz_manager"), "")
mgr.anonymous = true
-- auto reboot
reboot_stranded = mgr:option(Value, "auto_reboot_after_stranded", translate("mz_reboot_stra"))
reboot_stranded.rmempty = true
reboot_stranded.optional = true
reboot_stranded:value("600", "600" .. translate("mz_auto_reboot_600", " (10 minute)"))
reboot_stranded:value("3600", "3600" .. translate("mz_auto_reboot_3600", " (1 hour)"))
reboot_stranded:value("86400", "86400" .. translate("mz_auto_reboot_86400", " (1 day)"))

reboot_noroute = mgr:option(Value, "auto_reboot_after_no_route_to_gw", translate("mz_reboot_nogw"))
reboot_noroute.rmempty = true
reboot_noroute.optional = true
reboot_noroute:value("600", "600" .. translate("mz_auto_reboot_600", " (10 minute)"))
reboot_noroute:value("3600", "3600" .. translate("mz_auto_reboot_3600", " (1 hour)"))
reboot_noroute:value("86400", "86400" .. translate("mz_auto_reboot_86400", " (1 day)"))

-- dns resolv file
dns = mgr:option(ListValue, "dns_resolv_file", translate("mz_dns_resolv"))
dns:value("/tmp/resolv.conf.auto", "/tmp/resolv.conf.auto")
dns:value("none", "none")

-- good neighbor list
mgr:option(DynamicList, "goodNbr_ip_address", translate("mz_goodnbr"), translate("mz_goodnbr_desc"))

---- miscellaneous valiables
misc = m:section(TypedSection, "misc", translate("mz_misc"), translate("mz_misc_desc"))
misc.anonymous = true
misc:option(Value, "wlan_rts_threshold", "Wlan Rts Threshold").rmempty = false
misc:option(Value, "wlan_xmit_queue_length", "Wlan Xmit Queue Length").rmempty = false
misc:option(Value, "reliable_broadcast", "Reliable Broadcast").rmempty = false
misc:option(Value, "hello_by_unicast", "Hello By Unicast").rmempty = false
misc:option(Value, "link_quality_threshold", "Link Quality Threshold").rmempty = false

subnet_mode = misc:option(ListValue, "stub_subnet_mode", "Stub Subnet Mode")
subnet_mode:value("auto", "auto")
subnet_mode:value("manual", "manual")

stub_subnet = misc:option(Value, "stub_subnet", "Stub Subnet(manual)")
stub_subnet.rmempty = false
stub_subnet:depends("stub_subnet_mode", "manual")

------------------------------- validater ------------------------------
function name.validate(self, value)
	if value and #value > 0 and #value < 17 then
		return value
	else
		self.err_message = translate("mz_err_nodename")
		return nil
	end
end

function mz_ipaddr.validate(self, value)
	if value and nw.isIPv4(value) then
		return value
	else
		self.err_message = translate("mz_err_ipaddr")
		return nil
	end
end

function mz_subnet.validate(self, value)
	if value and value:match("/(.+)")
		and lip.IPv4(value) then
		return value
	else
		self.err_message = translate("mz_err_subnet")
		return nil
	end
end

function min_rssi.validate(self, value)
	local n = tonumber(value)
	if not n or n > 95 or n < 0 then
		self.err_message = translate("mz_err_rssi")
		return nil
	else
		return value
	end
end

function min_rate.validate(self, value)
    local n = tonumber(value)
	if not n or n > 54 or n < 0 then
		self.err_message = translate("mz_err_rate")
		return nil
	else
		return value
	end
end

pref_rate.validate = min_rate.validate
pref_rssi.validate = min_rssi.validate

--[[
function fs_subnet.validate(self, value)
	if value and lip.IPv4(value) then
		return value
	else
		self.err_message = translate("mz_err_fssubnet")
		return nil
	end
end

function fs_gwip.validate(self, value)
	if value and nw.isIPv4(value) then
		return value
	else
		self.err_message = translate("mz_err_ipaddr")
		return nil
	end
end
--]]

function reboot_stranded.validate(self, value)
	local n = tonumber(value)
	if not n or n < 0 then
		self.err_message = translate("mz_err_reboot")
		return nil
	else
		return value
	end
end

reboot_noroute.validate = reboot_stranded.validate

--[[
function goodnbr:validate(value)
	local tbl = value
	if type(tbl) == "string" then
		tbl = ttu.split(value, " ")
	end
	local ret = true
	for k, v in ipairs(tbl) do
		if not nw.isIPv4(v) then ret = false end
	end
	return ret and value or nil
end
--]]

return m
