
local util = require "luci.util"
local uci  = require "luci.model.uci".cursor()
local sys  = require "luci.sys"
local ver  = require "luci.version"
local web  = require "luci.tools.webadmin"
local nw   = require "luci.tools.network"
local nfs  = require "nixio.fs"
local ipairs, pairs, table, tonumber, string, io
		= ipairs, pairs, table, tonumber, string, io

module "luci.tools.ttutil"

function split(str, del)
   p, nrep = string.gsub(str, "%s*"..del.."%s*", "")
   return { str:match((("%s*(.-)%s*"..del.."%s*"):rep(nrep).."(.*)")) }
end

function get_ifaces4adhoc()
	local tbl = {}
    local devinfo = sys.net.deviceinfo()

	-- bridge interface
	for _, b in ipairs(nw.bridge_ifs()) do
		local br  = nw.get_brinfo(b)
		local dev = br and br.name
		if br and devinfo and devinfo[dev] then
			local e = {}
			e.ifname  = dev  -- br-LAN
			e.netname = dev  -- br-LAN
			tbl[#tbl+1] = e
		end
	end

	-- normal interface
	for _, i in ipairs(nw.single_ifs()) do
		local dev = nw.get_ifname(i)
		if dev and devinfo and devinfo[dev] and not nw.is_brport(dev) and
				not nw.is_wifi(dev) then
			local e = {}
			e.ifname = dev          -- ethX or wlanX
			e.netname = i['.name']  -- WAN, ADHOC_X, or AP_X
			tbl[#tbl+1] = e
		end
	end

	-- wifi interfaces
	for _, i in ipairs(nw.wireless_ifs()) do
		local dev = nw.get_ifname(i)
		local iw = nw.get_wlaninfo_by_ifname(dev)
		if dev and devinfo and devinfo[dev] and not nw.is_brport(dev) and nw.is_wifi(dev) and
				iw and iw.mode == "adhoc" then
			local e = {}
			e.ifname  = dev           -- ethX or wlanX
			e.radio   = i.device      -- RADIO_0, RADIO_1...
			e.netname = i.network     -- WAN, ADHOC_X, or AP_X
			tbl[#tbl+1] = e
		end
	end

	return tbl
end

function get_ifaces4exsubnet()
	local tbl = {}
    local devinfo = sys.net.deviceinfo()

	-- bridge interface
	for _, b in ipairs(nw.bridge_ifs()) do
		local br  = nw.get_brinfo(b)
		local dev = br and br.name
		if br and devinfo and devinfo[dev] then
			local e = {}
			e.ifname  = dev  -- br-LAN
			e.netname = dev  -- br-LAN (it's same on this case)
			e.ipaddr = b.ipaddr or ""
			e.netmask = b.netmask or ""
			tbl[#tbl+1] = e
		end
	end

	-- normal interface
	for _, i in ipairs(nw.single_ifs()) do
		local dev = nw.get_ifname(i)

		if dev and devinfo and devinfo[dev] and not nw.is_brport(dev) and
				not nw.is_wifi(dev) and i.ipaddr and i.netmask and i['.name'] then

			local e = {}
			e.ifname = dev          -- ethX or wlanX
			e.netname = i['.name']  -- WAN, ADHOC_X, or AP_X
			e.ipaddr = i.ipaddr --or ""
			e.netmask = i.netmask --or ""
			tbl[#tbl+1] = e

			-- search and add IP alias
			uci:foreach("network", "alias",
				function(s)
					if s.interface == e.netname and s.ipaddr and s.netmask then
						local ea = {}
						ea.ifname = "alias"
						ea.netname = s.interface
						ea.ipaddr = s.ipaddr
						ea.netmask = s.netmask
						tbl[#tbl+1] = ea
					end
				end)
		end
	end

	-- wifi interfaces
	for _, i in ipairs(nw.wireless_ifs()) do
		local dev = nw.get_ifname(i)
		local iw = nw.get_wlaninfo_by_ifname(dev)
		if dev and devinfo and devinfo[dev] and not nw.is_brport(dev) and nw.is_wifi(dev) and
				iw and iw.mode == "ap" then

			local ipinfo = nw.network_getinfo_by_network(i.network)

			if ipinfo.ipaddr and ipinfo.netmask and i.network then
				local e = {}
				e.ifname  = dev           -- ethX or wlanX
				e.radio   = i.device      -- RADIO_0, RADIO_1...
				e.netname = i.network     -- WAN, ADHOC_X, or AP_X
				e.ipaddr  = ipinfo.ipaddr --or ""
				e.netmask = ipinfo.netmask --or ""
				tbl[#tbl+1] = e

				-- search and add IP alias
				uci:foreach("network", "alias",
					function(s)
						if s.interface == e.netname and s.ipaddr and s.netmask then
							local ea = {}
							ea.ifname = "alias"
							ea.netname = s.interface
							ea.ipaddr = s.ipaddr
							ea.netmask = s.netmask
							tbl[#tbl+1] = ea
						end
					end)
			end
		end
	end

	return tbl
end

function firmware_version()
	local version = ver.distversion or "-"
	version = string.match(version, "%((.+)%)")
	if not version then
		version = ver.distversion
	end
	return version
end

function meshz_version()
	local mesh = util.exec("meshzd -v")
	if not mesh or #mesh < 1 then
		return "---"
	end
	local version = string.match(mesh, "%d+%.%d%.%d")
	local revision = string.match(mesh, "%[.+%]")
	revision = string.gsub(revision, "%[", "")
	revision = string.gsub(revision, "%]", "")
	return version..", "..revision
end

function rmr_hw_model()
	local _, model = sys.sysinfo()
	if string.sub(model, 1, 4) == "MIPS" then
		return "RMR-9000"
	else
		return "RMR-600"
	end
end

function check_meshz_license()
	local pattern = "MC_LICENSE="
	local fname = "/tmp/meshz_status"
	if not nfs.access(fname) then
		return ""
	end
	for line in io.lines(fname) do
		if string.find(line, pattern) then
			li = string.match(line, ".+=(.+)")
			if li and string.find(li, "RMR") then
				return li
			else
				return ""
			end
		end
	end
	return ""
end

function has_11j_card(phy)
	if not phy then
		return false
	end
	for _, f in ipairs(sys.wifi.channels(phy)) do
		if f.mhz >= 4920 and f.mhz <= 4980 then
			return true
		end
	end
	return false
end

function get_ifmac456(ifname)
	if not ifname or #ifname == 0 then
		return nil
	end
	return util.exec("LANG=C ifconfig "..ifname.. " | grep HWaddr | awk -F: '{print $5$6$7}'")
end
